--===========================================================
-- stingray_bitmap_sanitizer.ms
-- Contact: martin.ashton@autodesk.com
--
-- Usage: Scripting > Run Script > stingray_bitmap_sanitizer.ms
--
-- Copies any bitmaps containing a "." in their filename into the 
-- specified directory, and replaces the "." with "_", so the files
-- can be made compatible with Stingray.
--===========================================================
(
	local OVERWRITE_EXISTING = true -- Determines whether or not to overwrite existing files.
	local SILENT = false -- set this to true to show the result of each copy attempt.
	
	-- Get the save path the user wants and remember it.
	global SANITIZER_REMEMBERED_PATH
	local SANITIZER_SAVE_PATH
	
	-- Get all the bitmap textures associated to a given object.
	fn GetBitmapTextures theObjects = (
		-- From: http://www.scriptspot.com/forums/3ds-max/general-scripting/resource-get-all-textures-function
		texMaps = #()
		for obj in theObjects do
		(
			join texMaps (getClassInstances bitmapTexture target:obj asTrackViewPick:off)
		)
		makeUniqueArray texMaps
	)
	
	-- Determines if the file exists.
	fn FileExists filePath = (
		(getfiles filePath).count != 0
	)
	
	-- If the filename of the bitmap contains a ".", copies the file into the selected directory and replaces the 
	-- occurrences "." with underscores "_".
	fn SanitizeBitmapForStingray texMap = (		
		
		local result = undefined
		
		fullPath = texMap.filename --filenameFromPath texMap.filename
		filename = getFilenameFile fullPath
		extension = getFilenameType fullPath
		
		-- New material name - substitute periods "." with underscores "_"
		newFilename = substituteString filename "." "_"
		
		-- If no changes are required to the filename then return.
		if newFilename == filename then return result
		
		-- Rename the original filename
		fromFile = fullPath
		toFile = SANITIZER_SAVE_PATH + @"\" + newFilename + extension
		
		-- Delete the existing file if it exists.
		if (FileExists toFile) and OVERWRITE_EXISTING then deleteFile toFile
		
		-- Copy the file to the desired location.
		
		if copyFile fromFile toFile == false then (
			-- do nothing.
			print ("fromFile: " + fromFile)
			print ("toFile: " + toFile)
			print "COPY FAILED"
		) else (
			result = toFile
		)
		
		-- Finally, if the output file exists, assign it to the texture map.
		if (FileExists toFile) then (
			texMap.fileName = toFile
			texMap.reload()
		)
		
		result
	)
	
	-- Get all the bitmaps and sanitize them for stingray by replacing any "." in their filenames with "_"
	fn DoSanitization = (
		texMaps = (GetBitmapTextures (objects as array))
		
		copied = #()
		for t in texMaps do (
			local result = SanitizeBitmapForStingray t
			if result != undefined then append copied result
		)
		copied = makeUniqueArray copied
		
		if SILENT == false then (
			for c in copied do (
				print ("Remapped bitmap to: " + c)
			)
		)
	)
	
	-- If the save path is valid, perform the sanitization.
	-- Initialize the remembered path
	if SANITIZER_REMEMBERED_PATH == undefined then (
		SANITIZER_REMEMBERED_PATH = pathConfig.getCurrentProjectFolder()
	)
	
	-- Get the user-defined save file path for copied bitmaps.
	SANITIZER_SAVE_PATH = getSavePath caption:"Select a directory in which to save the bitmap copies whose filenames do not contain a period."  initialDir:SANITIZER_REMEMBERED_PATH
	
	-- Remember the path and do the work.
	if SANITIZER_SAVE_PATH != undefined then (
		SANITIZER_REMEMBERED_PATH = SANITIZER_SAVE_PATH
		DoSanitization()
		print "Stingray bitmap sanitation complete."
	) else (
		print "Stingray bitmap sanitation aborted."
	)
	
	ok
)